@@ -51,6 +51,7 @@
 static void gst_alsa_class_init (gpointer g_class, gpointer class_data);
 static void gst_alsa_init (GstAlsa * this);
 static void gst_alsa_dispose (GObject * object);
+static void gst_alsa_finalize (GObject * object);
 static void gst_alsa_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
 static void gst_alsa_get_property (GObject * object,
@@ -161,6 +162,7 @@ gst_alsa_class_init (gpointer g_class, gpointer class_data)
     parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
   object_class->dispose = gst_alsa_dispose;
+  object_class->finalize = gst_alsa_finalize;
   object_class->get_property = gst_alsa_get_property;
   object_class->set_property = gst_alsa_set_property;
 
@@ -218,14 +220,23 @@ gst_alsa_dispose (GObject * object)
 {
   GstAlsa *this = GST_ALSA (object);
 
-  g_free (this->device);
-
-  if (this->clock)
+  if (this->clock) {
     gst_object_unparent (GST_OBJECT (this->clock));
+    this->clock = NULL;
+  }
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
+static void
+gst_alsa_finalize (GObject * object)
+{
+  GstAlsa *this = GST_ALSA (object);
+
+  g_free (this->device);
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 static void
 gst_alsa_set_property (GObject * object, guint prop_id, const GValue * value,
     GParamSpec * pspec)
@@ -695,6 +706,7 @@ gst_alsa_get_caps_internal (snd_pcm_format_t format)
       return NULL;
 
     return gst_caps_new_simple ("audio/x-raw-float",
+        "buffer-frames", GST_TYPE_INT_RANGE, 0, G_MAXINT,
         "width", G_TYPE_INT, (gint) snd_pcm_format_width (format),
         "endianness", G_TYPE_INT, G_BYTE_ORDER, NULL);
   }
@@ -1088,9 +1100,6 @@ gst_alsa_change_state (GstElement * element)
         return GST_STATE_FAILURE;
       break;
     case GST_STATE_READY_TO_PAUSED:
-      if (!(GST_FLAG_IS_SET (element, GST_ALSA_RUNNING) ||
-              gst_alsa_start_audio (this)))
-        return GST_STATE_FAILURE;
       this->played = 0;
       this->captured = 0;
       break;
@@ -1101,8 +1110,14 @@ gst_alsa_change_state (GstElement * element)
               snd_strerror (err));
           return GST_STATE_FAILURE;
         }
-      } else if (!(GST_FLAG_IS_SET (element, GST_ALSA_RUNNING) ||
-              gst_alsa_start_audio (this))) {
+      }
+      /* If we were already negotiated, but we are not running, then
+       * we stopped (probably because we paused), so re-start. If
+       * there's no format, we didn't negotiate yet so don't do
+       * anything because ALSA will crash (#151288, #153227, etc.). */
+      else if (this->format != NULL &&
+          !GST_FLAG_IS_SET (element, GST_ALSA_RUNNING) &&
+          !gst_alsa_start_audio (this)) {
         return GST_STATE_FAILURE;
       }
       gst_alsa_clock_start (this->clock);
@@ -1180,8 +1195,9 @@ inline gboolean
 gst_alsa_pcm_wait (GstAlsa * this)
 {
   int err;
+  snd_pcm_state_t state = snd_pcm_state (this->handle);
 
-  if (snd_pcm_state (this->handle) == SND_PCM_STATE_RUNNING) {
+  if (state == SND_PCM_STATE_RUNNING) {
     if ((err = snd_pcm_wait (this->handle, 1000)) < 0) {
       if (err == EINTR) {
         /* happens mostly when run under gdb, or when exiting due to a signal */
@@ -1198,6 +1214,9 @@ gst_alsa_pcm_wait (GstAlsa * this)
         return FALSE;
       }
     }
+  } else {
+    GST_INFO_OBJECT (this, "in state %s, not waiting",
+        snd_pcm_state_name (state));
   }
   return TRUE;
 }
