@@ -56,6 +56,7 @@ enum
 static void gst_flacdec_base_init (gpointer g_class);
 static void gst_flacdec_class_init (FlacDecClass * klass);
 static void gst_flacdec_init (FlacDec * flacdec);
+static void gst_flacdec_finalize (GObject * object);
 
 static void gst_flacdec_loop (GstElement * element);
 static GstElementStateReturn gst_flacdec_change_state (GstElement * element);
@@ -112,6 +113,7 @@ flacdec_get_type (void)
       0,
       (GInstanceInitFunc) gst_flacdec_init,
     };
+
     flacdec_type =
         g_type_register_static (GST_TYPE_ELEMENT, "FlacDec", &flacdec_info, 0);
   }
@@ -167,6 +169,8 @@ gst_flacdec_class_init (FlacDecClass * klass)
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
+  gobject_class->finalize = gst_flacdec_finalize;
+
   gstelement_class->change_state = gst_flacdec_change_state;
 }
 
@@ -226,6 +230,21 @@ gst_flacdec_init (FlacDec * flacdec)
   FLAC__seekable_stream_decoder_set_client_data (flacdec->decoder, flacdec);
 }
 
+static void
+gst_flacdec_finalize (GObject * object)
+{
+  FlacDec *flacdec;
+
+  flacdec = GST_FLACDEC (object);
+
+  if (flacdec->decoder)
+    FLAC__seekable_stream_decoder_delete (flacdec->decoder);
+  flacdec->decoder = NULL;
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+
 static gboolean
 gst_flacdec_update_metadata (FlacDec * flacdec,
     const FLAC__StreamMetadata * metadata)
@@ -256,7 +275,8 @@ gst_flacdec_update_metadata (FlacDec * flacdec,
       g_free (value);
     }
   }
-
+  gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
+      GST_TAG_AUDIO_CODEC, "FLAC", NULL);
 
   gst_element_found_tags (GST_ELEMENT (flacdec), list);
   if (GST_PAD_IS_USABLE (flacdec->srcpad)) {
@@ -448,26 +468,24 @@ gst_flacdec_write (const FLAC__SeekableStreamDecoder * decoder,
 
     flacdec->need_discont = FALSE;
 
-    if (!GST_PAD_CAPS (flacdec->srcpad)) {
-      if (flacdec->seek_pending) {
-        flacdec->total_samples = flacdec->seek_value;
-      }
+    if (flacdec->seek_pending) {
+      flacdec->total_samples = flacdec->seek_value;
+    }
 
-      if (GST_PAD_IS_USABLE (flacdec->srcpad)) {
-        GST_DEBUG ("send discont");
+    if (GST_PAD_IS_USABLE (flacdec->srcpad)) {
+      GST_DEBUG ("send discont");
 
-        format = GST_FORMAT_TIME;
-        gst_pad_convert (flacdec->srcpad, GST_FORMAT_DEFAULT,
-            flacdec->total_samples, &format, &time);
-        format = GST_FORMAT_BYTES;
-        gst_pad_convert (flacdec->srcpad, GST_FORMAT_DEFAULT,
-            flacdec->total_samples, &format, &bytes);
-        discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time,
-            GST_FORMAT_BYTES, bytes,
-            GST_FORMAT_DEFAULT, flacdec->total_samples, NULL);
+      format = GST_FORMAT_TIME;
+      gst_pad_convert (flacdec->srcpad, GST_FORMAT_DEFAULT,
+          flacdec->total_samples, &format, &time);
+      format = GST_FORMAT_BYTES;
+      gst_pad_convert (flacdec->srcpad, GST_FORMAT_DEFAULT,
+          flacdec->total_samples, &format, &bytes);
+      discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time,
+          GST_FORMAT_BYTES, bytes,
+          GST_FORMAT_DEFAULT, flacdec->total_samples, NULL);
 
-        gst_pad_push (flacdec->srcpad, GST_DATA (discont));
-      }
+      gst_pad_push (flacdec->srcpad, GST_DATA (discont));
     }
   }
 
@@ -492,6 +510,8 @@ gst_flacdec_write (const FLAC__SeekableStreamDecoder * decoder,
     GST_BUFFER_DATA (outbuf) = g_malloc (GST_BUFFER_SIZE (outbuf));
     GST_BUFFER_TIMESTAMP (outbuf) =
         flacdec->total_samples * GST_SECOND / frame->header.sample_rate;
+    GST_BUFFER_DURATION (outbuf) =
+        samples * GST_SECOND / frame->header.sample_rate;
 
     if (depth == 8) {
       guint8 *outbuffer = (guint8 *) GST_BUFFER_DATA (outbuf);
@@ -585,6 +605,7 @@ gst_flacdec_get_src_formats (GstPad * pad)
     GST_FORMAT_TIME,
     0,
   };
+
   return formats;
 }
 
@@ -659,6 +680,7 @@ gst_flacdec_get_src_query_types (GstPad * pad)
     GST_QUERY_POSITION,
     0,
   };
+
   return types;
 }
 
@@ -679,12 +701,12 @@ gst_flacdec_src_query (GstPad * pad, GstQueryType type,
       else
         samples = flacdec->stream_samples;
 
-      gst_pad_convert (flacdec->srcpad,
+      res = gst_pad_convert (flacdec->srcpad,
           GST_FORMAT_DEFAULT, samples, format, value);
       break;
     }
     case GST_QUERY_POSITION:
-      gst_pad_convert (flacdec->srcpad,
+      res = gst_pad_convert (flacdec->srcpad,
           GST_FORMAT_DEFAULT, flacdec->total_samples, format, value);
       break;
     default:
@@ -702,6 +724,7 @@ gst_flacdec_get_src_event_masks (GstPad * pad)
     {GST_EVENT_SEEK, GST_SEEK_FLAG_ACCURATE},
     {0, 0},
   };
+
   return masks;
 }
 
