@@ -74,8 +74,9 @@ struct _GstMad
   GstTagList *tags;
 
   /* negotiated format */
-  gint rate;
-  gint channels;
+  gint rate, pending_rate;
+  gint channels, pending_channels;
+  gint times_pending;
 
   gboolean caps_set;            /* used to keep track of whether to change/update caps */
   GstIndex *index;
@@ -712,7 +713,8 @@ normal_seek (GstMad * mad, GstPad * pad, GstEvent * event)
       GstEvent *seek_event;
 
       /* conversion succeeded, create the seek */
-      seek_event = gst_event_new_seek (format | GST_SEEK_METHOD_SET | flush,
+      seek_event =
+          gst_event_new_seek (format | GST_EVENT_SEEK_METHOD (event) | flush,
           desired_offset);
       /* do the seek */
       if (gst_pad_send_event (GST_PAD_PEER (mad->sinkpad), seek_event)) {
@@ -981,7 +983,7 @@ gst_mad_check_restart (GstMad * mad)
  * This code has been kindly relicensed to LGPL by Thibaut Mattern and 
  * Bastien Nocera
  */
-#define BE_32(x) GINT32_FROM_BE(*(gint*)(x))
+#define BE_32(x) GST_READ_UINT32_BE(x)
 
 #define FOURCC_TAG( ch0, ch1, ch2, ch3 )		\
 	( (long)(unsigned char)(ch3) |			\
@@ -1152,7 +1154,16 @@ gst_mad_check_caps_reset (GstMad * mad)
       GST_DEBUG
           ("Header changed from %d Hz/%d ch to %d Hz/%d ch, failed sync after seek ?",
           mad->rate, mad->channels, rate, nchannels);
-      return;
+      /* we're conservative on stream changes. However, our *initial* caps
+       * might have been wrong as well - mad ain't perfect in syncing. So,
+       * we count caps changes and change if we pass a limit treshold (3). */
+      if (nchannels != mad->pending_channels || rate != mad->pending_rate) {
+        mad->times_pending = 0;
+        mad->pending_channels = nchannels;
+        mad->pending_rate = rate;
+      }
+      if (++mad->times_pending < 3)
+        return;
     }
   }
   gst_mad_update_info (mad);
@@ -1172,11 +1183,12 @@ gst_mad_check_caps_reset (GstMad * mad)
         "depth", G_TYPE_INT, 16,
         "rate", G_TYPE_INT, rate, "channels", G_TYPE_INT, nchannels, NULL);
 
-    gst_pad_set_explicit_caps (mad->srcpad, caps);
+    if (gst_pad_set_explicit_caps (mad->srcpad, caps)) {
+      mad->caps_set = TRUE;     /* set back to FALSE on discont */
+      mad->channels = nchannels;
+      mad->rate = rate;
+    }
     gst_caps_free (caps);
-    mad->caps_set = TRUE;       /* set back to FALSE on discont */
-    mad->channels = nchannels;
-    mad->rate = rate;
   }
 }
 
@@ -1234,7 +1246,15 @@ gst_mad_chain (GstPad * pad, GstData * _data)
     gint tocopy;
     guchar *mad_input_buffer;   /* convenience pointer to tempbuffer */
 
-    tocopy = MIN (MAD_BUFFER_MDLEN, size);
+    tocopy =
+        MIN (MAD_BUFFER_MDLEN, MIN (size,
+            MAD_BUFFER_MDLEN * 3 - mad->tempsize));
+    if (tocopy == 0) {
+      GST_ELEMENT_ERROR (mad, STREAM, DECODE, (NULL),
+          ("mad claims to need more data than %u bytes, we don't have that much",
+              MAD_BUFFER_MDLEN * 3));
+      return;
+    }
 
     /* append the chunk to process to our internal temporary buffer */
     GST_LOG ("tempbuffer size %d, copying %d bytes from incoming buffer",
@@ -1257,14 +1277,32 @@ gst_mad_chain (GstPad * pad, GstData * _data)
       unsigned char const *before_sync, *after_sync;
 
       mad->in_error = FALSE;
+
       mad_stream_buffer (&mad->stream, mad_input_buffer, mad->tempsize);
 
+      /* added separate header decoding to catch errors earlier, also fixes
+       * some weird decoding errors... */
+      GST_LOG ("decoding the header now");
+      if (mad_header_decode (&mad->frame.header, &mad->stream) == -1) {
+        GST_DEBUG ("mad_frame_decode had an error: %s",
+            mad_stream_errorstr (&mad->stream));
+      }
+
+      GST_LOG ("decoding one frame now");
+
       if (mad_frame_decode (&mad->frame, &mad->stream) == -1) {
+        GST_LOG ("got error %d", mad->stream.error);
+
         /* not enough data, need to wait for next buffer? */
         if (mad->stream.error == MAD_ERROR_BUFLEN) {
-          GST_LOG ("not enough data in tempbuffer (%d), breaking to get more",
-              mad->tempsize);
-          break;
+          if (mad->stream.next_frame == mad_input_buffer) {
+            GST_LOG ("not enough data in tempbuffer (%d), breaking to get more",
+                mad->tempsize);
+            break;
+          } else {
+            GST_LOG ("sync error, flushing unneeded data");
+            goto next;
+          }
         }
         /* we are in an error state */
         mad->in_error = TRUE;
@@ -1492,6 +1530,7 @@ gst_mad_change_state (GstElement * element)
       mad->rate = 0;
       mad->channels = 0;
       mad->caps_set = FALSE;
+      mad->times_pending = mad->pending_rate = mad->pending_channels = 0;
       mad->vbr_average = 0;
       mad->segment_start = 0;
       mad->new_header = TRUE;
