@@ -52,7 +52,7 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-raw-float, "
-        "rate = (int) [ 11025, 48000 ], "
+        "rate = (int) [ 8000, 50000 ], "
         "channels = (int) [ 1, 2 ], " "endianness = (int) BYTE_ORDER, "
 /* no ifdef in macros, please
 #ifdef GST_VORBIS_DEC_SEQUENTIAL
@@ -113,7 +113,7 @@ vorbis_dec_get_formats (GstPad * pad)
     0
   };
   static GstFormat sink_formats[] = {
-    GST_FORMAT_BYTES,
+    /*GST_FORMAT_BYTES, */
     GST_FORMAT_TIME,
     GST_FORMAT_DEFAULT,         /* granulepos or samples */
     0
@@ -153,6 +153,7 @@ gst_vorbis_dec_init (GstVorbisDec * dec)
       (&vorbis_dec_sink_factory), "sink");
   gst_pad_set_chain_function (dec->sinkpad, vorbis_dec_chain);
   gst_pad_set_formats_function (dec->sinkpad, vorbis_dec_get_formats);
+  gst_pad_set_convert_function (dec->sinkpad, vorbis_dec_convert);
   gst_element_add_pad (GST_ELEMENT (dec), dec->sinkpad);
 
   dec->srcpad =
@@ -184,6 +185,10 @@ vorbis_dec_convert (GstPad * pad,
   if (dec->packetno < 1)
     return FALSE;
 
+  if (dec->sinkpad == pad &&
+      (src_format == GST_FORMAT_BYTES || *dest_format == GST_FORMAT_BYTES))
+    return FALSE;
+
   switch (src_format) {
     case GST_FORMAT_TIME:
       switch (*dest_format) {
@@ -240,7 +245,8 @@ vorbis_dec_src_query (GstPad * pad, GstQueryType query, GstFormat * format,
     granulepos = dec->granulepos;
   } else {
     /* query peer in default format */
-    if (!gst_pad_query (GST_PAD_PEER (dec->sinkpad), query, &my_format,
+    if (!dec->sinkpad ||
+        !gst_pad_query (GST_PAD_PEER (dec->sinkpad), query, &my_format,
             &granulepos))
       return FALSE;
   }
@@ -367,6 +373,9 @@ vorbis_dec_chain (GstPad * pad, GstData * data)
   packet.granulepos = GST_BUFFER_OFFSET_END (buf);
   packet.packetno = vd->packetno++;
 
+  if (GST_BUFFER_OFFSET_END_IS_VALID (buf))
+    vd->granulepos = GST_BUFFER_OFFSET_END (buf);;
+
   /* 
    * FIXME. Is there anyway to know that this is the last packet and
    * set e_o_s??
@@ -378,9 +387,9 @@ vorbis_dec_chain (GstPad * pad, GstData * data)
     /* header packet */
     if (packet.packet[0] / 2 != packet.packetno) {
       /* FIXME: just skip? */
-      GST_ELEMENT_ERROR (GST_ELEMENT (vd), STREAM, DECODE,
-          (NULL), ("unexpected packet type %d, expected %d",
-              (gint) packet.packet[0], (gint) packet.packetno));
+      GST_WARNING_OBJECT (GST_ELEMENT (vd),
+          "unexpected packet type %d, expected %d",
+          (gint) packet.packet[0], (gint) packet.packetno);
       gst_data_unref (data);
       return;
     }
@@ -408,7 +417,8 @@ vorbis_dec_chain (GstPad * pad, GstData * data)
         g_free (encoder);
       }
       gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
-          GST_TAG_ENCODER_VERSION, vd->vi.version, NULL);
+          GST_TAG_ENCODER_VERSION, vd->vi.version,
+          GST_TAG_AUDIO_CODEC, "Vorbis", NULL);
       if (vd->vi.bitrate_upper > 0)
         gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
             GST_TAG_MAXIMUM_BITRATE, (guint) vd->vi.bitrate_upper, NULL);
