@@ -1170,6 +1170,8 @@ gst_asf_demux_process_chunk (GstASFDemux * asf_demux,
     stream->sequence = segment_info->sequence;
     asf_demux->pts = segment_info->frag_timestamp - asf_demux->preroll;
     got_bytes = gst_bytestream_peek (bs, &buffer, segment_info->chunk_size);
+    if (got_bytes == 0)
+      goto done;
     GST_DEBUG ("BUFFER: Copied stream to buffer (%p - %d)", buffer,
         GST_BUFFER_REFCOUNT_VALUE (buffer));
     stream->payload = buffer;
@@ -1185,6 +1187,8 @@ gst_asf_demux_process_chunk (GstASFDemux * asf_demux,
       /* continuing packet */
       GST_INFO ("A continuation packet");
       got_bytes = gst_bytestream_peek (bs, &buffer, segment_info->chunk_size);
+      if (got_bytes == 0)
+        goto done;
       GST_DEBUG ("Copied stream to buffer (%p - %d)", buffer,
           GST_BUFFER_REFCOUNT_VALUE (buffer));
       new_buffer = gst_buffer_merge (stream->payload, buffer);
@@ -1223,6 +1227,11 @@ gst_asf_demux_process_chunk (GstASFDemux * asf_demux,
 
   if (stream->frag_offset < segment_info->segment_size) {
     /* We don't have the whole packet yet */
+  } else if (got_bytes < segment_info->chunk_size) {
+    /* We didn't get the entire chunk, so don't push it down the pipeline. */
+    gst_buffer_unref (stream->payload);
+    stream->payload = NULL;
+    stream->frag_offset = 0;
   } else {
     /* We have the whole packet now so we should push the packet to
        the src pad now. First though we should check if we need to do
