@@ -18,7 +18,7 @@
  */
 
 #ifndef RGB_OUT
-#define RGB_OUT(d, r, g, b) RGBA_OUT(d, r, g, b, 0xffU)
+#define RGB_OUT(d, r, g, b) RGBA_OUT(d, r, g, b, 0xff)
 #endif
 
 static void glue(yuv420p_to_, RGB_NAME)(AVPicture *dst, const AVPicture *src,
@@ -256,6 +256,7 @@ static void glue(RGB_NAME, _to_yuv420p)(AVPicture *dst, const AVPicture *src,
             cb[0] = RGB_TO_U_CCIR(r1, g1, b1, 2);
             cr[0] = RGB_TO_V_CCIR(r1, g1, b1, 2);
 
+
             cb++;
             cr++;
             p += -wrap3 + 2 * BPP;
@@ -467,6 +468,60 @@ static void glue(RGB_NAME, _to_rgba32)(AVPicture *dst, const AVPicture *src,
     }
 }
 
+static void ayuv4444_to_rgba32(AVPicture *dst, const AVPicture *src,
+                             int width, int height)
+{
+    uint8_t *s, *d, *d1, *s1;
+    int w, y, cb, cr, r_add, g_add, b_add;
+    uint8_t *cm = cropTbl + MAX_NEG_CROP;
+    unsigned int r, g, b, a;
+
+    d = dst->data[0];
+    s = src->data[0];
+    for(;height > 0; height --) {
+        d1 = d;
+        s1 = s;
+        for(w = width; w > 0; w--) {
+	    a = s1[0];
+            YUV_TO_RGB1_CCIR(s1[2], s1[3]);
+
+            YUV_TO_RGB2_CCIR(r, g, b, s1[1]);
+            RGBA_OUT(d1, r, g, b, a);
+            d1 += BPP;
+            s1 += 4;
+        }
+        d += dst->linesize[0];
+        s += src->linesize[0];
+    }
+}
+
+static void rgba32_to_ayuv4444(AVPicture *dst, const AVPicture *src,
+                             int width, int height)
+{
+    int src_wrap, dst_wrap, x, y;
+    int r, g, b, a;
+    uint8_t *d;
+    const uint8_t *p;
+
+    src_wrap = src->linesize[0] - width * BPP;
+    dst_wrap = dst->linesize[0] - width * 4;
+    d = dst->data[0];
+    p = src->data[0];
+    for(y=0;y<height;y++) {
+        for(x=0;x<width;x++) {
+            RGBA_IN(r, g, b, a, p);
+	    d[0] = a;
+            d[1] = RGB_TO_Y_CCIR(r, g, b);
+            d[2] = RGB_TO_U_CCIR(r, g, b, 0);
+            d[3] = RGB_TO_V_CCIR(r, g, b, 0);
+            p += BPP;
+	    d += 4;
+        }
+        p += src_wrap;
+        d += dst_wrap;
+    }
+}
+
 #endif /* !defined(FMT_RGBA32) && defined(RGBA_IN) */
 
 #ifndef FMT_RGB24
@@ -768,6 +823,58 @@ static void rgb24_to_yuvj444p(AVPicture *dst, const AVPicture *src,
     }
 }
 
+static void ayuv4444_to_rgb24(AVPicture *dst, const AVPicture *src,
+                             int width, int height)
+{
+    uint8_t *s, *d, *d1, *s1;
+    int w, y, cb, cr, r_add, g_add, b_add;
+    uint8_t *cm = cropTbl + MAX_NEG_CROP;
+    unsigned int r, g, b;
+
+    d = dst->data[0];
+    s = src->data[0];
+    for(;height > 0; height --) {
+        d1 = d;
+        s1 = s;
+        for(w = width; w > 0; w--) {
+            YUV_TO_RGB1_CCIR(s1[2], s1[3]);
+
+            YUV_TO_RGB2_CCIR(r, g, b, s1[1]);
+            RGB_OUT(d1, r, g, b);
+            d1 += BPP;
+            s1 += 4;
+        }
+        d += dst->linesize[0];
+        s += src->linesize[0];
+    }
+}
+
+static void rgb24_to_ayuv4444(AVPicture *dst, const AVPicture *src,
+                             int width, int height)
+{
+    int src_wrap, dst_wrap, x, y;
+    int r, g, b;
+    uint8_t *d;
+    const uint8_t *p;
+
+    src_wrap = src->linesize[0] - width * BPP;
+    dst_wrap = dst->linesize[0] - width * 4;
+    d = dst->data[0];
+    p = src->data[0];
+    for(y=0;y<height;y++) {
+        for(x=0;x<width;x++) {
+            RGB_IN(r, g, b, p);
+	    d[0] = 0xff;
+            d[1] = RGB_TO_Y_CCIR(r, g, b);
+            d[2] = RGB_TO_U_CCIR(r, g, b, 0);
+            d[3] = RGB_TO_V_CCIR(r, g, b, 0);
+            p += BPP;
+	    d += 4;
+        }
+        p += src_wrap;
+        d += dst_wrap;
+    }
+}
 #endif /* FMT_RGB24 */
 
 #if defined(FMT_RGB24) || defined(FMT_RGBA32)
