@@ -60,6 +60,13 @@ enum
 
 /* Define the capabilities separately, to be able to reuse them. */
 
+static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("video/mpeg, "
+        "mpegversion = (int) 2, " "systemstream = (boolean) TRUE")
+    );
+
 #define VIDEO_CAPS \
   GST_STATIC_CAPS ("video/mpeg, " \
     "mpegversion = (int) { 1, 2 }, " \
@@ -73,7 +80,7 @@ enum
     "audio/x-raw-int, " \
       "endianness = (int) BIG_ENDIAN, " \
       "signed = (boolean) TRUE, " \
-      "width = (int) { 16, 20, 24 }, " \
+      "width = (int) { 16, 24 }, " \
       "depth = (int) { 16, 20, 24 }, " \
       "rate = (int) { 48000, 96000 }, " \
       "channels = (int) [ 1, 8 ];" \
@@ -196,6 +203,10 @@ gst_dvd_demux_base_init (GstDVDDemuxClass * klass)
 
   mpeg_parse_class->send_data = gst_dvd_demux_send_data;
 
+  /* sink pad */
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
+
   demux_class->audio_template = gst_static_pad_template_get (&audio_template);
 
   klass->cur_video_template = gst_static_pad_template_get (&cur_video_template);
@@ -247,7 +258,6 @@ static void
 gst_dvd_demux_init (GstDVDDemux * dvd_demux)
 {
   GstMPEGDemux *mpeg_demux = GST_MPEG_DEMUX (dvd_demux);
-  GstMPEGParse *mpeg_parse = GST_MPEG_PARSE (dvd_demux);
   gint i;
 
   GST_FLAG_SET (dvd_demux, GST_ELEMENT_EVENT_AWARE);
@@ -256,12 +266,15 @@ gst_dvd_demux_init (GstDVDDemux * dvd_demux)
   dvd_demux->cur_video =
       DEMUX_CLASS (dvd_demux)->new_output_pad (mpeg_demux, "current_video",
       CLASS (dvd_demux)->cur_video_template);
+  gst_element_add_pad (GST_ELEMENT (mpeg_demux), dvd_demux->cur_video);
   dvd_demux->cur_audio =
       DEMUX_CLASS (dvd_demux)->new_output_pad (mpeg_demux, "current_audio",
       CLASS (dvd_demux)->cur_audio_template);
+  gst_element_add_pad (GST_ELEMENT (mpeg_demux), dvd_demux->cur_audio);
   dvd_demux->cur_subpicture =
       DEMUX_CLASS (dvd_demux)->new_output_pad (mpeg_demux, "current_subpicture",
       CLASS (dvd_demux)->cur_subpicture_template);
+  gst_element_add_pad (GST_ELEMENT (mpeg_demux), dvd_demux->cur_subpicture);
 
   dvd_demux->mpeg_version = 0;
   dvd_demux->cur_video_nr = 0;
@@ -271,10 +284,24 @@ gst_dvd_demux_init (GstDVDDemux * dvd_demux)
   /* Start the timestamp sequence in 0. */
   dvd_demux->last_end_ptm = 0;
 
+  /* (Ronald) so, this was disabled. I'm enabling (default) it again.
+   * Timestamp adjustment is fairly evil, we would ideally use discont
+   * events instead. However, our current clocking has a pretty serious
+   * race condition: imagine that $pipeline is at time 30sec and $audio
+   * receives a discont to 0sec. Video processes its last buffer and
+   * calls _wait() on $timestamp, which is 30s - so we wait (hang) 30sec.
+   * This is unacceptable, obviously, and timestamp adjustment, no matter
+   * how evil, solves this.
+   * Before disabling this again, tripple check that al .vob files on our
+   * websites /media/ directory work fine, especially bullet.vob and
+   * barrage.vob.
+   */
+#if 0
   /* Try to prevent the mpegparse infrastructure from doing timestamp
      adjustment. */
   mpeg_parse->do_adjust = FALSE;
   mpeg_parse->adjust = 0;
+#endif
 
   dvd_demux->just_flushed = FALSE;
   dvd_demux->discont_time = GST_CLOCK_TIME_NONE;
@@ -484,9 +511,9 @@ gst_dvd_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
   guint8 sample_info = 0;
   GstMPEGStream *str;
   GstDVDLPCMStream *lpcm_str = NULL;
-  gchar *name;
   GstCaps *caps;
   gint width, rate, channels;
+  gboolean add_pad = FALSE;
 
   g_return_val_if_fail (stream_nr < GST_MPEG_DEMUX_NUM_AUDIO_STREAMS, NULL);
   g_return_val_if_fail (type > GST_MPEG_DEMUX_AUDIO_UNKNOWN &&
@@ -501,20 +528,23 @@ gst_dvd_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
   }
 
   str = mpeg_demux->audio_stream[stream_nr];
-
   if (str == NULL) {
+    gchar *name;
+
     if (type != GST_DVD_DEMUX_AUDIO_LPCM) {
       str = g_new0 (GstMPEGStream, 1);
     } else {
       lpcm_str = g_new0 (GstDVDLPCMStream, 1);
       str = (GstMPEGStream *) lpcm_str;
     }
-    str->type = GST_MPEG_DEMUX_AUDIO_UNKNOWN;
 
     name = g_strdup_printf ("audio_%02d", stream_nr);
     DEMUX_CLASS (dvd_demux)->init_stream (mpeg_demux, type, str, stream_nr,
         name, DEMUX_CLASS (dvd_demux)->audio_template);
+    /* update caps */
+    str->type = GST_MPEG_DEMUX_AUDIO_UNKNOWN;
     g_free (name);
+    add_pad = TRUE;
 
     mpeg_demux->audio_stream[stream_nr] = str;
   } else {
@@ -587,6 +617,8 @@ gst_dvd_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
       /* This is the current audio stream.  Use the same caps. */
       gst_pad_set_explicit_caps (dvd_demux->cur_audio, gst_caps_copy (caps));
     }
+    if (add_pad)
+      gst_element_add_pad (GST_ELEMENT (mpeg_demux), str->pad);
 
     str->type = type;
   }
@@ -603,6 +635,7 @@ gst_dvd_demux_get_subpicture_stream (GstMPEGDemux * mpeg_demux,
   GstMPEGStream *str;
   gchar *name;
   GstCaps *caps;
+  gboolean add_pad = FALSE;
 
   g_return_val_if_fail (stream_nr < GST_DVD_DEMUX_NUM_SUBPICTURE_STREAMS, NULL);
   g_return_val_if_fail (type > GST_DVD_DEMUX_SUBP_UNKNOWN &&
@@ -612,12 +645,13 @@ gst_dvd_demux_get_subpicture_stream (GstMPEGDemux * mpeg_demux,
 
   if (str == NULL) {
     str = g_new0 (GstMPEGStream, 1);
-    str->type = GST_DVD_DEMUX_SUBP_UNKNOWN;
 
     name = g_strdup_printf ("subpicture_%02d", stream_nr);
     DEMUX_CLASS (dvd_demux)->init_stream (mpeg_demux, type, str, stream_nr,
         name, CLASS (dvd_demux)->subpicture_template);
+    str->type = GST_DVD_DEMUX_SUBP_UNKNOWN;
     g_free (name);
+    add_pad = TRUE;
 
     dvd_demux->subpicture_stream[stream_nr] = str;
   } else {
@@ -637,6 +671,8 @@ gst_dvd_demux_get_subpicture_stream (GstMPEGDemux * mpeg_demux,
     }
 
     gst_caps_free (caps);
+    if (add_pad)
+      gst_element_add_pad (GST_ELEMENT (mpeg_demux), str->pad);
     str->type = GST_DVD_DEMUX_SUBP_DVD;
   }
 
@@ -850,7 +886,7 @@ gst_dvd_demux_set_cur_audio (GstDVDDemux * dvd_demux, gint stream_nr)
   str = mpeg_demux->audio_stream[stream_nr];
   if (str != NULL) {
     /* (Re)set the caps in the "current" pad. */
-    caps = gst_pad_get_negotiated_caps (str->pad);
+    caps = GST_RPAD_EXPLICIT_CAPS (str->pad);
     if (caps != NULL) {
       gst_pad_set_explicit_caps (dvd_demux->cur_audio, caps);
     }
@@ -862,7 +898,6 @@ static void
 gst_dvd_demux_set_cur_subpicture (GstDVDDemux * dvd_demux, gint stream_nr)
 {
   GstMPEGStream *str;
-  const GstCaps *caps = NULL;
 
   g_return_if_fail (stream_nr >= -1 &&
       stream_nr < GST_DVD_DEMUX_NUM_SUBPICTURE_STREAMS);
@@ -878,11 +913,11 @@ gst_dvd_demux_set_cur_subpicture (GstDVDDemux * dvd_demux, gint stream_nr)
 
   str = dvd_demux->subpicture_stream[stream_nr];
   if (str != NULL) {
+    GstCaps *caps = NULL;
+
     /* (Re)set the caps in the "current" pad. */
-    caps = gst_pad_get_negotiated_caps (str->pad);
-    if (caps != NULL) {
-      gst_pad_set_explicit_caps (dvd_demux->cur_subpicture, caps);
-    }
+    caps = GST_RPAD_EXPLICIT_CAPS (str->pad);
+    gst_pad_set_explicit_caps (dvd_demux->cur_subpicture, caps);
   }
 }
 
@@ -891,5 +926,5 @@ gboolean
 gst_dvd_demux_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "dvddemux",
-      GST_RANK_PRIMARY - 1, GST_TYPE_DVD_DEMUX);
+      GST_RANK_PRIMARY, GST_TYPE_DVD_DEMUX);
 }
